import numpy as np
import json

import numpy as np

class TSCyclic(object):
    #этот вариант работает по отсечке таймстампа, окно берется с запасом
    def __init__(self, maxlen=1000000):
        self.maxlen = maxlen
        self.shift = 0        
        self.ts = np.zeros(maxlen)
        self.vals = np.zeros(maxlen)  
        
    def add(self, timestamp, val):
        self.vals[self.shift], self.ts[self.shift] = val, timestamp
        self.shift = (self.shift + 1) % self.maxlen
        
    def get_older_then(self, timestamp):
        return self.vals[np.flatnonzero(self.ts >= timestamp)]

    def get_quantiles(self, timestamp, quantiles=[0.33, 0.66, 1]):
        vals = self.get_older_then(timestamp)
        return np.quantile(vals, quantiles, interpolation='nearest')   
    
    def get_quant_counts(self, timestamp, quantiles=[0.2, 0.4, 0.6, 0.8, 1]):
        vals = self.get_older_then(timestamp)
        return np.quantile(vals, quantiles, interpolation='nearest')*len(vals)
    
    def get_hist_counts(self, timestamp, bins, rng=None):
        vals = self.get_older_then(timestamp)
        if rng:
            return np.histogram(vals, bins=bins, range=rng)[0]
        else:
            return np.histogram(vals, bins=bins)[0]
    
class Cyclic(object):
    #этот вариант работает по всему буферу, размер которого задается заранее
    def __init__(self, win=300):
        self.win = win
        self.shift = 0
        self.vals = np.full(win, np.nan) 
        
    def add(self, val):
        self.vals[self.shift] = val
        self.shift = (self.shift + 1) % self.win
        
    def get_valid_vals(self):
        return self.vals[~np.isnan(self.vals)]
        
    def get_quantiles(self, quantiles=[0.33, 0.66, 1]):
        vals = np.nanquantile(self.vals, quantiles, interpolation='nearest')
        return vals
    
    def get_quant_counts(self, quantiles=[0.2, 0.4, 0.6, 0.8, 1]):
        vals = np.nanquantile(self.vals, quantiles, interpolation='nearest')
        return vals * np.count_nonzero(~np.isnan(data))
    
class PSITS(object):
    #http://ucanalytics.com/blogs/population-stability-index-psi-banking-case-study/
    #этот вариант работает по отсечке таймстампа, окно берется с запасом
    def __init__(self, expected_distribution_vector, bins=10, maxlen=1000000):        
        self.n_const = np.full(bins, 0.5)
        self.hist = lambda x: np.histogram(x, bins=bins, range=(0.0, 1.0))[0] + self.n_const

        self.buf = TSCyclic(maxlen)
        self.add = self.buf.add
        
        expected = self.hist(np.array(expected_distribution_vector))
        self.expected = expected/expected.sum()
        
    def compute(self, timestamp):
        raw_hist = self.hist(self.buf.get_older_then(timestamp))
        actual = raw_hist/raw_hist.sum()
        terms = (actual - self.expected)*np.log(actual/self.expected)
        return terms.sum() #percents
        
class PSI(object):
    #http://ucanalytics.com/blogs/population-stability-index-psi-banking-case-study/
    #этот вариант работает по всему буферу, размер которого задается заранее
    def __init__(self, expected_distribution_vector, bins=10, win=1000):        
        self.n_const = np.full(bins, 0.5)
        self.hist = lambda x: np.histogram(x, bins=bins, range=(0.0, 1.0))[0] + self.n_const

        self.buf = Cyclic(win)
        self.add = self.buf.add
        
        expected = self.hist(np.array(expected_distribution_vector))
        self.expected = expected/expected.sum()
        
    def compute(self):
        raw_hist = self.hist(self.buf.get_valid_vals())
        actual = raw_hist/raw_hist.sum()
        terms = (actual - self.expected)*np.log(actual/self.expected)
        return terms.sum() #percents    

'''    
class PSIFabric(object):
    def __init__(self):
        self.psi_dict = {}
        #если есть в конфиге имя и версия пайплайна, то будет для него создана PSI
    def set_expected_values_files(self, expected_values_files):
        #структура такая expected_values_files[pipeline_name][pipeline_ver] = file_path
        #эта штука не апдейтит, только создает, можно повтороно запускать с каждой конфигой, уже сконфигуренное оно не трогает
        psi_dict = self.psi_dict
        for pipeline_name in expected_values_files:            
            if pipeline_name not in psi_dict:
                psi_dict[pipeline_name] = {}
                
            for pipeline_ver in expected_values_files[pipeline_name]:
                if pipeline_ver in psi_dict[pipeline_name]:
                    continue
                expected_values_fname = expected_values_files[pipeline_name][pipeline_ver]
                with open(expected_values_fname) as 
                psi_dict[pipeline_name][pipeline_ver] = PSI(expected_values)

                
    def add_and_compute(pipeline_name, pipeline_ver, timestamp, val, time_range):
        if pipeline_name not in self.psi_dict:
            return None
        if pipeline_ver not in self.psi_dict[pipeline_name]:
            return None
        psi = self.psi_dict[pipeline_name][pipeline_ver]
        psi.add(timestamp, val)
        return psi.compute(timestamp-time_range)
'''    