from .actor import MessageBus, Actor, Logger
import json, csv

import uuid
from time import time, sleep
import traceback

from threading import Thread


def from_json_gen(filename):
    '''
    [
      {"field1": "val11", "field2": "val12"},
      {"field1": "val21", "field2": "val22"},
      {"field1": "val31", "field2": "val32"}
    ]
    '''
    with open(filename, 'r') as f:
        rows = json.load(f)
    for dct in rows:
        yield dct
        
def from_ndjson_gen(filename):
    '''
    {"field1": "val11", "field2": "val12"}
    {"field1": "val21", "field2": "val22"}
    {"field1": "val31", "field2": "val32"}
    {"field1": "val\n41", "field2": "val4\n2"}          
    '''  
    with open(filename, 'r') as f: 
        for line in f:
            yield json.loads(line)
        
def from_headless_csv_gen(filename):
    '''
    val11; val12
    val21; val22
    val31; val32
    '''      
    with open(filename, 'r') as f: 
        for row in csv.reader(f):
            yield row
                    
def from_csv_gen(filename, row_as_dict=True):
    '''
    field1; field2
    val11; val12
    val21; val22
    val31; val32
    ''' 
    with open(filename, 'r') as f:        
        if row_as_dict:
            reader = csv.DictReader(file_like, *args, **kwags)
        else:
            csvreader = csv.reader(file_like, *args, **kwags)
            head = next(csvreader)
            reader = csvreader    
        
        for row in reader:
            yield row

class Api(Actor):
    
    def __init__(self, store,
                       in_topics = ['fabrique.api.in', ], 
                       monitor_topics = ['fabrique.monitor.in',],
                       pipeline_name = 'pipeline_name',
                       pipeline_ver='pipeline_ver',
                       input_generator = None,
                       file_out_name = 'pipeline_out.ndjson',
                       session_timeout = 60.0
                       ):
        
        self.store = store
        
        if not input_generator:
            input_generator = from_ndjson_gen('pipeline_in.ndjson')
        
        self.session_ids = set()
        self.last_time_sent = time()
        self.session_timeout = session_timeout
        
        self.file_in = None
        self.file_out_name = file_out_name
        self.file_out = open(self.file_out_name, 'w')
        
        self.input_generator = input_generator

        self.metric_fields = []
        self.client_id = 'api'
        self.in_topics = in_topics
        self.pipeline_ver = pipeline_ver
        self.pipeline_name = pipeline_name

        #logger
        self.logger = Logger.make_logger(self.client_id)
        #1. лист входных топиков или один топик - неважно
        self.in_topics = in_topics
        #2. вЫходной топик для данных, результатов, скоров и т.д.
        self.out_topic = f'fabrique.api.{pipeline_name}.out'
        #3. топик для ошибок
        self.error_topic = in_topics[0]
        #4. топик для мониторинга
        self.monitor_topics = monitor_topics
    
    def send_messages2pipeline(self):
        for data in self.input_generator:
            session_id = str(uuid.uuid4())
            body = {'data': data}

            self.send_data(body, session_id)
            
            self.session_ids.add(session_id)
            self.last_time_sent = time()
            
        while self.session_ids:            
            if (time() - self.last_time_sent) > self.session_timeout:
                self.store.stop = True
                print('! Timeout for {} sessions'.format(len(self.session_ids))) 
                break                
            sleep(0.1)
            
        if not self.session_ids:
            print(f'All sessions have been processed (file {self.file_out_name})')
        
        sleep(5) #мониторинги не успевают иногда   
        self.close_all()
        
    def close_all(self):
        self.store.stop = True
        self.file_out.close()
 
    def batch_callback(self, bodies):
        return bodies
        
    def process_result(self, res_body, session_id):
        #print(res_body)
        self.file_out.write(json.dumps(res_body)+'\n')
        
        self.session_ids.remove(session_id)
        
    def run_data(self):
        api_sender = Thread(target=self.send_messages2pipeline)
        api_receiver = Thread(target=self.run)
        
        api_receiver.start()
        api_sender.start()
        api_receiver.join()
        api_sender.join()
        
        


            