from .actor import Actor
import json

def_config = {
    "message_bus": {  
      "pipeline_name": "all",
      "pipeline_ver": "all",
      "in_topic": "fabrique.monitor.in",
      "client_id": "monitor",
      "error_topic": None,
      "group_id": None,
      "servers": None,
      }
  }

class Monitor(Actor):
    def __init__(self, store, file_out_name = 'metrics.ndjson'):   
        config = def_config
        config['message_bus']['servers'] = store
        self.file_out_name = file_out_name
        self.file_out = open(self.file_out_name, 'w')
        
        super().__init__(config)        
    
    """
        Сборщик данных мониторинга
        ничего не отдает, принимает месседжи всех типов, при получении 
    """
    def preprocess_msg(self, msg):
        message = super().preprocess_msg(msg)
        if message:
            #раз колбеку доступен только боди для нужд менеджмента закинем в боди все что может быть интересно
            header = message.header.to_dict()
            kafka_header = message.kafka_header
            body = message.body
            session_id = message.header.session_id
            message.body = dict(body=body, kafka_header=kafka_header, header=header, session_id=session_id)
            return message
 
    def close_all(self):
        self.file_out.close()

    def process_result(self, res_body, session_id):
        return None
    
    def batch_callback(self, batch):
        for message_dict in batch:
            self.callback(message_dict)
    
    def callback(self, message_dict): 
        
        
        session_id = message_dict["session_id"]     
        client_id = message_dict["header"]["source"] #сlient_id актора, приславшего метрики
        source_id = client_id.rsplit('__')[0] #отсечем суффикс клона
        timestamp = message_dict["kafka_header"]["timestamp"] 

        pipeline_name = message_dict["header"]["pipeline_name"]
        pipeline_ver = message_dict["header"]["pipeline_ver"] 

        #if not pipeline_name and not pipeline_ver:
        #    return

        body = message_dict['body']
                
        def write_metric(field, pipeline_name, pipeline_ver, source_id, value=None):
            
            self.file_out.write(json.dumps(dict(field=field, value=value, pipeline_name=pipeline_name, 
                                                pipeline_ver=pipeline_ver, source_id=source_id))+'\n')


        #обновим метрики
        for field in body:
            #print('! metric', str(mval), field, pipeline_name, pipeline_ver, source_id)
            if field=="event": #вероятно какой-то енум ['start', ...]
                #пока не придумал, "start" будет первым в сессии, потому автоматом будет хвататься кодом для таймингов
                continue

            if field=="error": #текст мониторингу не интересен, а факт ошибки фиксируем
                write_metric(field, pipeline_name, pipeline_ver, source_id)
                
        #кастомные поля по умолчанию 
        if body.get('metrics'):

            #print('! metrics', body['metrics'], pipeline_name, pipeline_ver, source_id)

            for field, value in body['metrics'].items():
                #pri.set_custom_summary(value, field, pipeline_name, pipeline_ver, gauge_id)
                write_metric(field, pipeline_name, pipeline_ver, source_id, value)  

        #кастомные поля по умолчанию 
        if body.get('alerts'):

            #print('! metrics', body['metrics'], pipeline_name, pipeline_ver, source_id)

            for field, value in body['alerts'].items():
                #pri.set_custom_summary(value, field, pipeline_name, pipeline_ver, gauge_id)
                write_metric(field, pipeline_name, pipeline_ver, source_id, value)             

