try:
    import fabrique.actor as actor
except ImportError:
    import atelier_fabrique.dummy.actor as actor
    
from atelier_fabrique.constants import VALID_ACTOR_TYPES
    
import json
from collections import namedtuple

class BaseEngine():
    
    def __str__(self):
        actor_type = self._actor_type
        class_name = type(self).__name__
        
        if actor_type == 'dispatcher':
            output = list(self.destinations)
        else:
            output = self.__dict__.get('output')          
            
        return f'{class_name}, type = {actor_type}, output = {output}' 
    
    def set_output(self, output):
        self.output = output
        
    def set_destinations(self, *destinations):
        raise Exception('Must use set_output except set_destinations')
        
    def set_input(self, input_class):
        self.input = input_class
    
    def default_get_batch_result(self, batch):
        res_batch = []
        for body in batch:
            res_batch.append(self.get_result(body))
        return res_batch

    def get_result(self, data):
        raise Exception("This method needs to be defined in a subclass")
    
    def start_actor(self, actor_instance, autorun=True):
        self._actor = actor_instance

        try:
            self._actor.batch_callback = self.get_batch_result  #если определен такой метод, то фигачим батчевый колбек       
        except AttributeError:
            self._actor.batch_callback = self.default_get_batch_result
            
        if autorun:
            self._actor.run()
            
            
class Reducer(BaseEngine):
    _actor_type = VALID_ACTOR_TYPES.reducer    
    def start(self, config):
        self.start_actor(actor.Reducer(config))

class Mapper(BaseEngine):
    _actor_type = VALID_ACTOR_TYPES.mapper
    def start(self, config):
        self.start_actor(actor.Mapper(config))

class Dispatcher(BaseEngine):
    _actor_type = VALID_ACTOR_TYPES.dispatcher
    def set_output(self, output):
        raise Exception('Must use set_destinations except set_output')
    
    def set_destinations(self, destinations):
        self.destinations = namedtuple('ids', destinations)(*destinations)
        
    def start(self, config):
        self.start_actor(actor.Dispatcher(config))
        
class MetricsReducer(BaseEngine):
    _actor_type = VALID_ACTOR_TYPES.metrics_reducer    
    def start(self, config):
        self.start_actor(actor.Reducer(config))
        
class Nodes(dict):
    def __init__(self,**kw):
        dict.__init__(self,kw)
        self.__dict__ = self

class Pipeline(object):
    def __init__(self, nodes_lst):
        assert(nodes_lst)
        assert(not set(nodes_lst).intersection({'begin', 'end', 'monitor'})) 
        self.nodes = Nodes()
        for node_id in nodes_lst:
            setattr(self.nodes, node_id, None)        
        
        nodes_lst.extend(['begin', 'end'])  
        self.ids = namedtuple('ids', nodes_lst)(*nodes_lst)
        
